Class: SCTP::Socket
- Inherits:
-
Object
- Object
- SCTP::Socket
- Defined in:
- ext/sctp/socket.c
Defined Under Namespace
Constant Summary collapse
- VERSION =
The version of this library
0.0.1
Instance Attribute Summary collapse
Instance Method Summary collapse
-
#bind(*args) ⇒ Object
Bind a subset of IP addresses associated with the host system on the given port, or a port assigned by the operating system if none is provided.
-
#close ⇒ Object
Close the socket.
-
#connect(*args) ⇒ Object
Connect the socket to a multihomed peer via the provided array of addresses using the domain specified in the constructor.
- #getlocalnames ⇒ Object
- #getpeernames ⇒ Object
-
#initialize(*args) ⇒ Object
constructor
Create and return a new SCTP::Socket instance.
-
#listen(*args) ⇒ Object
Marks the socket referred to by sockfd as a passive socket, i.e.
-
#peeloff!(v_assoc_id) ⇒ Object
Extracts an association contained by a one-to-many socket connection into a one-to-one style socket.
-
#recvmsg(*args) ⇒ Object
Receive a message from another SCTP endpoint.
-
#sendmsg(v_options) ⇒ Object
Transmit a message to an SCTP endpoint.
-
#set_initmsg(v_options) ⇒ Object
{ :output_streams => 2, :input_streams => 3, :max_attempts => 5, :timeout => 30 }.
-
#subscribe(v_options) ⇒ Object
Subscribe to various notification types, which will generate additional data that the socket may receive.
Constructor Details
#initialize(*args) ⇒ Object
Create and return a new SCTP::Socket instance. You may optionally pass in a domain (aka family) value and socket type. By default these are AF_INET and SOCK_SEQPACKET, respectively.
Example:
socket1 = SCTP::Socket.new
socket2 = SCTP::Socket.new(Socket::AF_INET, Socket::SOCK_STREAM)
34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 |
# File 'ext/sctp/socket.c', line 34 static VALUE rsctp_init(int argc, VALUE* argv, VALUE self){ int sock_fd; VALUE v_domain, v_type; rb_scan_args(argc, argv, "02", &v_domain, &v_type); if(NIL_P(v_domain)) v_domain = INT2NUM(AF_INET); if(NIL_P(v_type)) v_type = INT2NUM(SOCK_SEQPACKET); sock_fd = socket(NUM2INT(v_domain), NUM2INT(v_type), IPPROTO_SCTP); if(sock_fd < 0) rb_raise(rb_eSystemCallError, "socket: %s", strerror(errno)); rb_iv_set(self, "@domain", v_domain); rb_iv_set(self, "@type", v_type); rb_iv_set(self, "@sock_fd", INT2NUM(sock_fd)); rb_iv_set(self, "@association_id", INT2NUM(0)); return self; } |
Instance Attribute Details
#association_id ⇒ Object
#domain ⇒ Object
#port ⇒ Object
#sock_fd ⇒ Object
#type ⇒ Object
Instance Method Details
#bind(*args) ⇒ Object
Bind a subset of IP addresses associated with the host system on the given port, or a port assigned by the operating system if none is provided.
Note that you can both add or remove an address to or from the socket using the SCTP_BINDX_ADD_ADDR (default) or SCTP_BINDX_REM_ADDR constants, respectively.
Example:
socket = SCTP::Socket.new
# Bind 2 addresses
socket.bind(:port => 64325, :addresses => ['10.0.4.5', '10.0.5.5'])
# Remove 1 later
socket.bind(:addresses => ['10.0.4.5'], :flags => SCTP::Socket::BINDX_REM_ADDR)
If no addresses are specified, then it will bind to all available interfaces. If no port is specified, then one will be assigned by the host.
Returns the port that it was bound to.
82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 |
# File 'ext/sctp/socket.c', line 82 static VALUE rsctp_bind(int argc, VALUE* argv, VALUE self){ struct sockaddr_in addrs[8]; int i, sock_fd, num_ip, flags, domain, port; VALUE v_addresses, v_port, v_flags, v_address, ; rb_scan_args(argc, argv, "01", &); bzero(&addrs, sizeof(addrs)); if(NIL_P()) = rb_hash_new(); v_addresses = rb_hash_aref2(, "addresses"); v_flags = rb_hash_aref2(, "flags"); v_port = rb_hash_aref2(, "port"); if(NIL_P(v_port)) port = 0; else port = NUM2INT(v_port); if(NIL_P(v_flags)) flags = SCTP_BINDX_ADD_ADDR; else flags = NUM2INT(v_flags); if(NIL_P(v_addresses)) num_ip = 1; else num_ip = RARRAY_LEN(v_addresses); domain = NUM2INT(rb_iv_get(self, "@domain")); sock_fd = NUM2INT(rb_iv_get(self, "@sock_fd")); if(num_ip > 1){ for(i = 0; i < num_ip; i++){ v_address = RARRAY_PTR(v_addresses)[i]; addrs[i].sin_family = domain; addrs[i].sin_port = htons(port); addrs[i].sin_addr.s_addr = inet_addr(StringValueCStr(v_address)); } } else{ addrs[0].sin_family = domain; addrs[0].sin_port = htons(port); addrs[0].sin_addr.s_addr = htonl(INADDR_ANY); } if(sctp_bindx(sock_fd, (struct sockaddr *) addrs, num_ip, flags) != 0) rb_raise(rb_eSystemCallError, "sctp_bindx: %s", strerror(errno)); if(port == 0){ struct sockaddr_in sin; socklen_t len = sizeof(sin); if(getsockname(sock_fd, (struct sockaddr *)&sin, &len) == -1) rb_raise(rb_eSystemCallError, "getsockname: %s", strerror(errno)); port = sin.sin_port; } return INT2NUM(port); } |
#close ⇒ Object
Close the socket. You should always do this.
Example:
socket = SCTP::Socket.new
socket.close
209 210 211 212 213 214 215 216 |
# File 'ext/sctp/socket.c', line 209 static VALUE rsctp_close(VALUE self){ VALUE v_sock_fd = rb_iv_get(self, "@sock_fd"); if(close(NUM2INT(v_sock_fd))) rb_raise(rb_eSystemCallError, "close: %s", strerror(errno)); return self; } |
#connect(*args) ⇒ Object
Connect the socket to a multihomed peer via the provided array of addresses using the domain specified in the constructor. You must also specify the port.
Example:
socket = SCTP::Socket.new
socket.connect(:port => 62354, :addresses => ['10.0.4.5', '10.0.5.5'])
Note that this will also set/update the object’s association_id.
157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 |
# File 'ext/sctp/socket.c', line 157 static VALUE rsctp_connect(int argc, VALUE* argv, VALUE self){ struct sockaddr_in addrs[8]; int i, num_ip, sock_fd; sctp_assoc_t assoc; VALUE v_address, v_domain, , v_addresses, v_port; rb_scan_args(argc, argv, "01", &); if(NIL_P()) rb_raise(rb_eArgError, "you must specify an array of addresses"); Check_Type(, T_HASH); v_addresses = rb_hash_aref2(, "addresses"); v_port = rb_hash_aref2(, "port"); if(NIL_P(v_addresses) || RARRAY_LEN(v_addresses) == 0) rb_raise(rb_eArgError, "you must specify an array of addresses containing at least one address"); if(NIL_P(v_port)) rb_raise(rb_eArgError, "you must specify a port"); v_domain = rb_iv_get(self, "@domain"); num_ip = RARRAY_LEN(v_addresses); bzero(&addrs, sizeof(addrs)); for(i = 0; i < num_ip; i++){ v_address = RARRAY_PTR(v_addresses)[i]; addrs[i].sin_family = NUM2INT(v_domain); addrs[i].sin_port = htons(NUM2INT(v_port)); addrs[i].sin_addr.s_addr = inet_addr(StringValueCStr(v_address)); } sock_fd = NUM2INT(rb_iv_get(self, "@sock_fd")); if(sctp_connectx(sock_fd, (struct sockaddr *) addrs, num_ip, &assoc) < 0) rb_raise(rb_eSystemCallError, "sctp_connectx: %s", strerror(errno)); rb_iv_set(self, "@assocation_id", INT2NUM(assoc)); return self; } |
#getlocalnames ⇒ Object
245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 |
# File 'ext/sctp/socket.c', line 245 static VALUE rsctp_getlocalnames(VALUE self){ sctp_assoc_t assoc_id; struct sockaddr* addrs; int i, sock_fd, num_addrs; bzero(&addrs, sizeof(addrs)); sock_fd = NUM2INT(rb_iv_get(self, "@sock_fd")); assoc_id = NUM2INT(rb_iv_get(self, "@assocation_id")); num_addrs = sctp_getladdrs(sock_fd, assoc_id, &addrs); if(num_addrs < 0){ sctp_freeladdrs(addrs); rb_raise(rb_eSystemCallError, "sctp_getladdrs: %s", strerror(errno)); } for(i = 0; i < num_addrs; i++){ // TODO: Create and return array of IpAddr objects } sctp_freeladdrs(addrs); return self; } |
#getpeernames ⇒ Object
218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 |
# File 'ext/sctp/socket.c', line 218 static VALUE rsctp_getpeernames(VALUE self){ VALUE v_assoc_id = rb_iv_get(self, "@assocation_id"); sctp_assoc_t assoc_id; struct sockaddr* addrs; int i, sock_fd, num_addrs; bzero(&addrs, sizeof(addrs)); sock_fd = NUM2INT(rb_iv_get(self, "@sock_fd")); assoc_id = NUM2INT(v_assoc_id); num_addrs = sctp_getpaddrs(sock_fd, assoc_id, &addrs); if(num_addrs < 0){ sctp_freepaddrs(addrs); rb_raise(rb_eSystemCallError, "sctp_getpaddrs: %s", strerror(errno)); } for(i = 0; i < num_addrs; i++){ // TODO: Create and return array of IpAddr objects } sctp_freepaddrs(addrs); return self; } |
#listen(*args) ⇒ Object
Marks the socket referred to by sockfd as a passive socket, i.e. a socket that will be used to accept incoming connection requests.
The backlog argument defines the maximum length to which the queue of pending connections for sockfd may grow. The default is 1024.
Example:
socket = SCTP::Socket.new
socket.bind(:port => 62534, :addresses => ['127.0.0.1'])
socket.listen
608 609 610 611 612 613 614 615 616 617 618 619 620 621 622 623 624 625 |
# File 'ext/sctp/socket.c', line 608 static VALUE rsctp_listen(int argc, VALUE* argv, VALUE self){ VALUE v_backlog; int backlog, sock_fd; rb_scan_args(argc, argv, "01", &v_backlog); if(NIL_P(v_backlog)) backlog = 1024; else backlog = NUM2INT(v_backlog); sock_fd = NUM2INT(rb_iv_get(self, "@sock_fd")); if(listen(sock_fd, backlog) < 0) rb_raise(rb_eSystemCallError, "setsockopt: %s", strerror(errno)); return self; } |
#peeloff!(v_assoc_id) ⇒ Object
Extracts an association contained by a one-to-many socket connection into a one-to-one style socket. Note that this modifies the underlying sock_fd.
631 632 633 634 635 636 637 638 639 640 641 642 643 644 645 646 |
# File 'ext/sctp/socket.c', line 631 static VALUE rsctp_peeloff(VALUE self, VALUE v_assoc_id){ int sock_fd, new_sock_fd; sctp_assoc_t assoc_id; sock_fd = NUM2INT(rb_iv_get(self, "@sock_fd")); assoc_id = NUM2INT(v_assoc_id); new_sock_fd = sctp_peeloff(sock_fd, assoc_id); if(new_sock_fd < 0) rb_raise(rb_eSystemCallError, "sctp_peeloff: %s", strerror(errno)); rb_iv_set(self, "@sock_fd", INT2NUM(new_sock_fd)); return self; } |
#recvmsg(*args) ⇒ Object
Receive a message from another SCTP endpoint.
Example:
begin
socket = SCTP::Socket.new
socket.bind(:port => 62534, :addresses => ['10.0.4.5', '10.0.5.5'])
socket.subscribe(:data_io => 1)
socket.listen
while true
info = socket.recvmsg
puts "Received message: #{info.message}"
end
ensure
socket.close
end
408 409 410 411 412 413 414 415 416 417 418 419 420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 444 445 446 447 448 449 450 451 452 453 454 455 456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 |
# File 'ext/sctp/socket.c', line 408 static VALUE rsctp_recvmsg(int argc, VALUE* argv, VALUE self){ VALUE v_flags; struct sctp_sndrcvinfo sndrcvinfo; struct sockaddr_in clientaddr; int flags, bytes, sock_fd; char buffer[1024]; // TODO: Let this be configurable? socklen_t length; rb_scan_args(argc, argv, "01", &v_flags); if(NIL_P(v_flags)) flags = 0; else flags = NUM2INT(v_flags); sock_fd = NUM2INT(rb_iv_get(self, "@sock_fd")); length = sizeof(struct sockaddr_in); bzero(buffer, sizeof(buffer)); bytes = sctp_recvmsg( sock_fd, buffer, sizeof(buffer), (struct sockaddr*)&clientaddr, &length, &sndrcvinfo, &flags ); if(bytes < 0) rb_raise(rb_eSystemCallError, "sctp_recvmsg: %s", strerror(errno)); // TODO: Check for MSG_NOTIFICATION, return different structs for events. /* if(flags & MSG_NOTIFICATION){ union sctp_notification* snp; snp = (union sctp_notification*)buffer; switch(snp->sn_type){ case SCTP_ASSOC_CHANGE: break; case SCTP_PEER_ADDR_CHANGE: break; case SCTP_REMOTE_ERROR: break; case SCTP_SEND_FAILED: break; case SCTP_SHUTDOWN_EVENT: break; case SCTP_ADAPTATION_INDICATION: break; case SCTP_PARTIAL_DELIVERY_EVENT: break; } } */ return rb_struct_new(v_sndrcv_struct, rb_str_new(buffer, bytes), UINT2NUM(sndrcvinfo.sinfo_stream), UINT2NUM(sndrcvinfo.sinfo_flags), UINT2NUM(sndrcvinfo.sinfo_ppid), UINT2NUM(sndrcvinfo.sinfo_context), UINT2NUM(sndrcvinfo.sinfo_timetolive), UINT2NUM(sndrcvinfo.sinfo_assoc_id) ); } |
#sendmsg(v_options) ⇒ Object
Transmit a message to an SCTP endpoint. The following hash of options is permitted:
:message -> The to send to the endpoint. Mandatory.
:stream -> The SCTP stream number you wish to send the on.
:to -> An array of addresses to send the to.
:context -> The default context used for the sendmsg call if the send fails.
:ppid -> The payload protocol identifier that is passed to the peer endpoint.
:flags -> A bitwise integer that contain one or more values that control behavior.
Note that the :to option is not mandatory in a one-to-one (SOCK_STREAM)
socket connection. However, it must have been set previously via the
connect method.
Example:
socket = SCTP::Socket.new
socket.sendmsg(
:message => "Hello World!",
:stream => 3,
:flags => SCTP::Socket::SCTP_UNORDERED | SCTP::Socket::SCTP_SENDALL,
:ttl => 100,
:to => ['10.0.5.4', '10.0.6.4']
)
298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 |
# File 'ext/sctp/socket.c', line 298 static VALUE rsctp_sendmsg(VALUE self, VALUE ){ VALUE v_msg, v_ppid, v_flags, v_stream, v_ttl, v_context, v_addresses; uint16_t stream; uint32_t ppid, flags, ttl, context; ssize_t num_bytes; struct sockaddr_in addrs[8]; int sock_fd, size; Check_Type(, T_HASH); bzero(&addrs, sizeof(addrs)); v_msg = rb_hash_aref2(, "message"); v_stream = rb_hash_aref2(, "stream"); v_ppid = rb_hash_aref2(, "ppid"); v_context = rb_hash_aref2(, "context"); v_flags = rb_hash_aref2(, "flags"); v_ttl = rb_hash_aref2(, "ttl"); v_addresses = rb_hash_aref2(, "addresses"); if(NIL_P(v_stream)) stream = 0; else stream = NUM2INT(v_stream); if(NIL_P(v_flags)) flags = 0; else flags = NUM2INT(v_stream); if(NIL_P(v_ttl)) ttl = 0; else ttl = NUM2INT(v_ttl); if(NIL_P(v_ppid)) ppid = 0; else ppid = NUM2INT(v_ppid); if(NIL_P(v_context)) context = 0; else context = NUM2INT(v_context); if(!NIL_P(v_addresses)){ int i, num_ip, port; VALUE v_address, v_port; num_ip = RARRAY_LEN(v_addresses); v_port = rb_hash_aref2(, "port"); if(NIL_P(v_port)) port = 0; else port = NUM2INT(v_port); for(i = 0; i < num_ip; i++){ v_address = RARRAY_PTR(v_addresses)[i]; addrs[i].sin_family = NUM2INT(rb_iv_get(self, "@domain")); addrs[i].sin_port = htons(port); addrs[i].sin_addr.s_addr = inet_addr(StringValueCStr(v_address)); } size = sizeof(addrs); } else{ size = 0; } sock_fd = NUM2INT(rb_iv_get(self, "@sock_fd")); num_bytes = sctp_sendmsg( sock_fd, StringValueCStr(v_msg), RSTRING_LEN(v_msg), (struct sockaddr*)addrs, size, ppid, flags, stream, ttl, context ); if(num_bytes < 0) rb_raise(rb_eSystemCallError, "sctp_sendmsg: %s", strerror(errno)); return INT2NUM(num_bytes); } |
#set_initmsg(v_options) ⇒ Object
:output_streams => 2,
:input_streams => 3,
:max_attempts => 5,
:timeout => 30
484 485 486 487 488 489 490 491 492 493 494 495 496 497 498 499 500 501 502 503 504 505 506 507 508 509 510 511 512 513 514 |
# File 'ext/sctp/socket.c', line 484 static VALUE rsctp_set_initmsg(VALUE self, VALUE ){ int sock_fd; struct sctp_initmsg initmsg; VALUE v_output, v_input, v_attempts, v_timeout; bzero(&initmsg, sizeof(initmsg)); v_output = rb_hash_aref2(, "output_streams"); v_input = rb_hash_aref2(, "input_streams"); v_attempts = rb_hash_aref2(, "max_attempts"); v_timeout = rb_hash_aref2(, "timeout"); sock_fd = NUM2INT(rb_iv_get(self, "@sock_fd")); if(!NIL_P(v_output)) initmsg.sinit_num_ostreams = NUM2INT(v_output); if(!NIL_P(v_input)) initmsg.sinit_max_instreams = NUM2INT(v_input); if(!NIL_P(v_attempts)) initmsg.sinit_max_attempts = NUM2INT(v_attempts); if(!NIL_P(v_timeout)) initmsg.sinit_max_init_timeo = NUM2INT(v_timeout); if(setsockopt(sock_fd, IPPROTO_SCTP, SCTP_INITMSG, &initmsg, sizeof(initmsg)) < 0) rb_raise(rb_eSystemCallError, "setsockopt: %s", strerror(errno)); return self; } |
#subscribe(v_options) ⇒ Object
Subscribe to various notification types, which will generate additional data that the socket may receive. The possible notification types are as follows:
:association
- A change has occurred to an assocation, either a new one has begun or an existing one has end.
:address
- The state of one of the peer's addresses has experienced a change.
:send_failure
- The message could not be delivered to a peer.
:shutdown
- The peer has sent a shutdown to the local endpoint.
Others:
:adaptation_layer
:authentication_event
:data_io
:peer_error
:partial_delivery
:sender_dry
By default only data_io is subscribed to.
Example:
socket = SCTP::Socket.new
socket.bind(:port => port, :addresses => ['127.0.0.1'])
socket.subscribe(:shutdown => true, :send_failure => true)
551 552 553 554 555 556 557 558 559 560 561 562 563 564 565 566 567 568 569 570 571 572 573 574 575 576 577 578 579 580 581 582 583 584 585 586 587 588 589 590 591 592 |
# File 'ext/sctp/socket.c', line 551 static VALUE rsctp_subscribe(VALUE self, VALUE ){ int sock_fd; struct sctp_event_subscribe events; bzero(&events, sizeof(events)); sock_fd = NUM2INT(rb_iv_get(self, "@sock_fd")); if(RTEST(rb_hash_aref2(, "data_io"))) events.sctp_data_io_event = 1; if(RTEST(rb_hash_aref2(, "association"))) events.sctp_association_event = 1; if(RTEST(rb_hash_aref2(, "address"))) events.sctp_address_event = 1; if(RTEST(rb_hash_aref2(, "send_failure"))) events.sctp_send_failure_event = 1; if(RTEST(rb_hash_aref2(, "peer_error"))) events.sctp_peer_error_event = 1; if(RTEST(rb_hash_aref2(, "shutdown"))) events.sctp_shutdown_event = 1; if(RTEST(rb_hash_aref2(, "partial_delivery"))) events.sctp_partial_delivery_event = 1; if(RTEST(rb_hash_aref2(, "adaptation_layer"))) events.sctp_adaptation_layer_event = 1; if(RTEST(rb_hash_aref2(, "authentication"))) events.sctp_authentication_event = 1; if(RTEST(rb_hash_aref2(, "sender_dry"))) events.sctp_sender_dry_event = 1; if(setsockopt(sock_fd, IPPROTO_SCTP, SCTP_EVENTS, &events, sizeof(events)) < 0) rb_raise(rb_eSystemCallError, "setsockopt: %s", strerror(errno)); return self; } |